/**
* LICENSE 
* Bonita
* Copyright (C) 2005 Bull S.A.
* Bull 68 route de versailles  78434 Louveciennes Cedex France
* Further information: bonita@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*/
package org.ow2.proed.chi.test;

import java.util.Iterator;
import java.util.List;
import java.util.Set;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.ow2.proed.Trace;
import org.ow2.proed.graphModel.AtomicActivity;
import org.ow2.proed.graphModel.BlockActivity;
import org.ow2.proed.graphModel.ProEdProject;
import org.ow2.proed.graphModel.RouteActivity;
import org.ow2.proed.graphModel.Subflow;
import org.ow2.proed.graphModel.Transition;
import org.ow2.proed.graphModel.WorkflowProcess;
import org.ow2.proed.graphModel.WorkflowProcesses;
import org.ow2.proed.graphModel.exception.TransitionException;
import org.ow2.proed.resources.PropertiesManager;


/**
 * CollectionElementTest.java Created on 21 juin 2005
 * 
 * @author Ducreux Evelyne - Gonthier Manu�la
 * @version $Id: CollectionElementTest.java,v 1.3 2005/08/04 15:18:46 gonthiem
 *          Exp $
 *          <p>
 */
public class CollectionElementTest extends TestCase
{

    private ProEdProject project;

    //trace for error
    private Trace trace;

    //process activities to test
    private AtomicActivity a1;

    private BlockActivity a2;

    private RouteActivity a3;

    private Subflow s;

    //transitions to test
    private Transition t1;

    private Transition t2;

    private Transition t3;

    private Transition t4;

    private PropertiesManager propManager = new PropertiesManager();

    public CollectionElementTest(String name)
    {
        super(name);
    }

    protected void setUp()
    {

        project = new ProEdProject("Project1", false);
        WorkflowProcesses workflows = new WorkflowProcesses();
        WorkflowProcess mainWorkflow = new WorkflowProcess("project1");
        project.setMyProcesses(workflows);
        workflows.add(mainWorkflow);

        //set up all process activities
        a1 = new AtomicActivity("Activit� 1",mainWorkflow);
        a2 = new BlockActivity("Activit� 2", project.getMyProcesses()
                .getMainWorkflow());
        a3 = new RouteActivity("Activit� 3",mainWorkflow);
        s = new Subflow("subflow 1",mainWorkflow);

        try
        {
            //set up good transitions
            t1 = new Transition(a1, a2, mainWorkflow);
            assertNotNull(t1);
            t2 = new Transition(a2, a3, mainWorkflow);
            assertNotNull(t2);
            t3 = new Transition(a1, s, mainWorkflow);
            assertNotNull(t3);
        }
        catch (TransitionException e2)
        {
            System.out.println(e2.getMessage());
        }
        trace = new Trace(true, true);

        project.getMyProcesses().getMainWorkflow().addProcessActivity(a1);
        project.getMyProcesses().getMainWorkflow().addProcessActivity(a2);
        project.getMyProcesses().getMainWorkflow().addProcessActivity(a3);
        project.getMyProcesses().getMainWorkflow().addProcessActivity(s);

        assertNotNull(project.getMyProcesses().getMainWorkflow()
                .getProcessActivities());
        assertEquals(project.getMyProcesses().getMainWorkflow()
                .getProcessActivities().size(), 4);

        project.getMyProcesses().getMainWorkflow().addTransitions(
                t1.getSource(), t1.getTarget());
        project.getMyProcesses().getMainWorkflow().addTransitions(
                t2.getSource(), t2.getTarget());
        project.getMyProcesses().getMainWorkflow().addTransitions(
                t3.getSource(), t3.getTarget());

        assertNotNull(project.getMyProcesses().getMainWorkflow()
                .getTransitions());
        assertEquals(project.getMyProcesses().getMainWorkflow()
                .getTransitions().size(), 3);

    }

    public void addElementinWorkflow(WorkflowProcess workflow)
    {
        try
        {

            //set up good transitions
            t4 = new Transition(a3, s, workflow);
            assertNotNull(t4);

        }
        catch (TransitionException e2)
        {
            System.out.println(e2.getMessage());
        }
        project.getMyProcesses().getMainWorkflow().addTransitions(
                t4.getSource(), t4.getTarget());

    }

    public void modifyT3(WorkflowProcess workflow)
    {
        try
        {
            addElementinWorkflow(workflow);
            project.getMyProcesses().getMainWorkflow().getTransition(t3)
                    .setSource(s);
            project.getMyProcesses().getMainWorkflow().getTransition(t3)
                    .setTarget(a1);

        }
        catch (TransitionException e2)
        {
            System.out.println(e2.getMessage());
        }
    }

    public void modifyT3Bis(WorkflowProcess workflow)
    {
        try
        {
            addElementinWorkflow(workflow);
            project.getMyProcesses().getMainWorkflow().getTransition(t3)
                    .setSource(s);
            project.getMyProcesses().getMainWorkflow().getTransition(t3)
                    .setTarget(a2);

        }
        catch (TransitionException e2)
        {
            System.out.println(e2.getMessage());
        }
    }

    public void testGetNextElement()
    {
        System.out.println("\n\n******testGetNextElementWithT4********");
        printTrajet();
        System.out.println("**************");
        //get all activities after a1
        List allActivities = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(a1);
        System.out.println("Activities after A1");
        for (Iterator iter = allActivities.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }

        //get all activities after a2
        List allActivitiesAfterA2 = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(a2);
        System.out.println("Activities after A2");
        for (Iterator iter = allActivitiesAfterA2.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }

        //      get all activities after a3
        List allActivitiesAfterA3 = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(a3);
        System.out.println("Activities after A3");
        for (Iterator iter = allActivitiesAfterA3.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }
    }

    public void testGetNextElementWithT4(WorkflowProcess workflow)
    {
        addElementinWorkflow(workflow);
        System.out.println("\n\n******testGetNextElementWithT4********");
        printTrajet();
        System.out.println("**************");
        //get all activities after a1
        List allActivities = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(a1);
        System.out.println("Activities after A1");
        for (Iterator iter = allActivities.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }

        //get all activities after a2
        List allActivitiesAfterA2 = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(a2);
        System.out.println("Activities after A2");
        for (Iterator iter = allActivitiesAfterA2.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }

        //      get all activities after a3
        List allActivitiesAfterA3 = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(a3);
        System.out.println("Activities after A3");
        for (Iterator iter = allActivitiesAfterA3.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }

        //      get all activities after s
        List allActivitiesAfterS = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(s);
        System.out.println("Activities after S");
        for (Iterator iter = allActivitiesAfterS.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }
    }

    private void printTrajet()
    {
        Set allTransitions = project.getMyProcesses().getMainWorkflow()
                .getTransitions();
        for (Iterator iter = allTransitions.iterator(); iter.hasNext();)
        {
            Transition t = (Transition) iter.next();
            System.out.println(t.getSource() + "->" + t.getTarget());
        }
    }

    public void testGetNextElementWithT3Modified(WorkflowProcess workflow)
    {
        //workflow like a1->a2->a3->s1->a1
        modifyT3(workflow);
        System.out.println("\n\n**************");
        printTrajet();
        System.out.println("**************");
        //get all activities after a1
        List allActivities = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(a1);
        System.out.println("Activities after A1");
        for (Iterator iter = allActivities.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }

        //get all activities after a2
        List allActivitiesAfterA2 = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(a2);
        System.out.println("Activities after A2");
        for (Iterator iter = allActivitiesAfterA2.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }

        //      get all activities after a3
        List allActivitiesAfterA3 = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(a3);
        System.out.println("Activities after A3");
        for (Iterator iter = allActivitiesAfterA3.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }

        //      get all activities after s
        List allActivitiesAfterS = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(s);
        System.out.println("Activities after S");
        for (Iterator iter = allActivitiesAfterS.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }
    }

    public void testGetNextElementWithT3ModifiedBis(WorkflowProcess workflow)
    {
        //workflow like a1->a2->a3->s1->a2
        modifyT3Bis(workflow);
        System.out
                .println("\n\n******testGetNextElementWithT3ModifiedBis********");
        printTrajet();
        System.out.println("**************");
        //get all activities after a1
        List allActivities = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(a1);
        System.out.println("Activities after A1");
        for (Iterator iter = allActivities.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }

        //get all activities after a2
        List allActivitiesAfterA2 = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(a2);
        System.out.println("Activities after A2");
        for (Iterator iter = allActivitiesAfterA2.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }

        //      get all activities after a3
        List allActivitiesAfterA3 = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(a3);
        System.out.println("Activities after A3");
        for (Iterator iter = allActivitiesAfterA3.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }

        //      get all activities after s
        List allActivitiesAfterS = project.getMyProcesses().getMainWorkflow()
                .getNextActivities(s);
        System.out.println("Activities after S");
        for (Iterator iter = allActivitiesAfterS.iterator(); iter.hasNext();)
        {
            System.out.println(iter.next());
        }
    }

    public static Test suite()
    {
        return new TestSuite(CollectionElementTest.class);
    }

}