/**
* LICENSE 
* Bonita
* Copyright (C) 2005 Bull S.A.
* Bull 68 route de versailles  78434 Louveciennes Cedex France
* Further information: bonita@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*/
package org.ow2.proed.chi.test;

import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.jdom.Document;
import org.ow2.proed.Trace;
import org.ow2.proed.Utils;
import org.ow2.proed.chi.mainFrame.MainFrame;
import org.ow2.proed.graphModel.ProEdProject;
import org.ow2.proed.graphModel.ProcessActivity;
import org.ow2.proed.graphModel.Subflow;
import org.ow2.proed.graphModel.WorkflowProcess;
import org.ow2.proed.graphModel.WorkflowProcesses;
import org.ow2.proed.xpdl.XPDLProEdProject;

/**
 * ImportSubflowTest.java
 * Created on 22 juin 2005
 * 
 * @author Ducreux Evelyne - Gonthier Manu�la
 * @version $Id: ImportSubflowTest.java 3505 2008-09-18 13:16:10Z vigneras $
 * <p>
 */
public class ImportSubflowTest extends TestCase
{
    //  trace for error
    private Trace trace;

    private String fileName = "TestJUnit.xpdl";

    public final static String SUFFIX_ID_CHANGED = "_IMPORT";

    private ProEdProject project;
    
    WorkflowProcess mainWorkflow;

    private Subflow subflow;

    protected void setUp()
    {
        trace = new Trace(true, true);

        project = new ProEdProject("Project1", false);
        WorkflowProcesses workflows = new WorkflowProcesses();
        mainWorkflow = new WorkflowProcess("project1");
        project.setMyProcesses(workflows);
        workflows.add(mainWorkflow);
    }

    public void testImportSubflow()
    {
    	String[] args = new String[0];
    	MainFrame owner = new MainFrame(true,true,args);
        Document doc=null;
        try
        {
            doc = Utils.loadXMLDocument(fileName);
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        assertNotNull(doc);
        
        if (doc != null)
        {
            //          Import this file in a temporary project
            ProEdProject projectImported = XPDLProEdProject.toModel(doc,owner);
            assertNotNull(projectImported);

            subflow = new Subflow(projectImported.getName(), projectImported
                    .getDescription(), ProcessActivity.TYPE_AND_JOIN,mainWorkflow);
            subflow.setWorkflow(projectImported.getMyProcesses()
                    .getMainWorkflow());
            assertNotNull(subflow);

            //add all workflow processes in this process
            project.getMyProcesses().addAll(projectImported.getMyProcesses());
            assertEquals(project.getMyProcesses().size()-1, projectImported
                    .getMyProcesses().size());

            //add this subflow in main workflow
            project.getMyProcesses().getMainWorkflow().addProcessActivity(
                    subflow);
            assertEquals(project.getMyProcesses().getMainWorkflow()
                    .getProcessActivities().size(), 1);

        }

    }
    
    public static Test suite()
    {
        return new TestSuite(ImportSubflowTest.class);
    }
}