/**
* LICENSE 
* Bonita
* Copyright (C) 2005 Bull S.A.
* Bull 68 route de versailles  78434 Louveciennes Cedex France
* Further information: bonita@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*/
package org.ow2.proed.chi.test;

import java.io.FileOutputStream;
import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.jdom.Document;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.ow2.proed.Trace;
import org.ow2.proed.Utils;
import org.ow2.proed.chi.mainFrame.MainFrame;
import org.ow2.proed.graphModel.AtomicActivity;
import org.ow2.proed.graphModel.BlockActivity;
import org.ow2.proed.graphModel.Participant;
import org.ow2.proed.graphModel.ProEdProject;
import org.ow2.proed.graphModel.RouteActivity;
import org.ow2.proed.graphModel.Subflow;
import org.ow2.proed.graphModel.Transition;
import org.ow2.proed.graphModel.WorkflowProcess;
import org.ow2.proed.graphModel.WorkflowProcesses;
import org.ow2.proed.graphModel.exception.ParticipantTypeException;
import org.ow2.proed.graphModel.exception.TransitionException;
import org.ow2.proed.resources.PropertiesManager;
import org.ow2.proed.xpdl.XPDLProEdProject;
import org.ow2.proed.xpdl.XPDLProperty;


/**
 * XPDLtest.java Created on 21 juin 2005
 * 
 * @author Ducreux Evelyne - Gonthier Manu�la
 * @version $Id: XPDLtest.java 3505 2008-09-18 13:16:10Z vigneras $
 *          <p>
 */
public class XPDLtest extends TestCase
{

    private ProEdProject project;

    private XPDLProEdProject xpdlProject;

    private String fileName = "TestJUnit.xpdl";

    //trace for error
    private Trace trace;

    //process activities to test
    private AtomicActivity a1;

    private BlockActivity a2;

    private RouteActivity a3;

    private Subflow s;

    //transitions to test
    private Transition t1;

    private Transition t2;

    private Participant participant;

    private PropertiesManager propManager = new PropertiesManager();

    public XPDLtest(String name)
    {
        super(name);
    }

    protected void setUp()
    {

        project = new ProEdProject("Project1", false);
        WorkflowProcesses workflows = new WorkflowProcesses();
        WorkflowProcess mainWorkflow = new WorkflowProcess("project1");
        project.setMyProcesses(workflows);
        workflows.add(mainWorkflow);

        //set up all process activities
        a1 = new AtomicActivity("Activit� 1",mainWorkflow);
        a2 = new BlockActivity("Activit� 2", project.getMyProcesses()
                .getMainWorkflow());
        a3 = new RouteActivity("Activit� 3",mainWorkflow);
        s = new Subflow("subflow 1",mainWorkflow);

        //set up all transitions
        try
        {
            //set up bad transitions
            t1 = new Transition(null, a2, mainWorkflow);
            assertNull(t1);

            t1 = new Transition(null, null, mainWorkflow);
            assertNull(t1);

            t1 = new Transition(a1, null, mainWorkflow);
            assertNull(t1);

        }
        catch (TransitionException e1)
        {
            System.out.println(e1.getMessage());
        }
        try
        {
            //set up good transitions
            t1 = new Transition(a1, a2, mainWorkflow);
            assertNotNull(t1);
            t2 = new Transition(a2, a3, mainWorkflow);
            assertNotNull(t2);
        }
        catch (TransitionException e2)
        {
            System.out.println(e2.getMessage());
        }

        try
        {
            //set up bad participant
            participant = new Participant("Manue", null);
            assertNull(participant);
            participant = new Participant("Manue", "manue", mainWorkflow);
            assertNull(participant);

        }
        catch (ParticipantTypeException e)
        {

            System.out.println(e.getMessage());
        }

        try
        {
            //set up good participant
            participant = new Participant("Manue", Participant.CST_HUMAN, mainWorkflow);
            assertNotNull(participant);
        }
        catch (ParticipantTypeException e)
        {
            System.out.println(e.getMessage());
        }

        trace = new Trace(true, true);
    }

    public void testAddingOfActivities()
    {
        project.getMyProcesses().getMainWorkflow().addProcessActivity(a1);
        project.getMyProcesses().getMainWorkflow().addProcessActivity(a2);
        project.getMyProcesses().getMainWorkflow().addProcessActivity(a3);
        project.getMyProcesses().getMainWorkflow().addProcessActivity(s);

        assertNotNull(project.getMyProcesses().getMainWorkflow()
                .getProcessActivities());
        assertEquals(project.getMyProcesses().getMainWorkflow()
                .getProcessActivities().size(), 4);

    }

    public void testAddingOfTransition()
    {
        project.getMyProcesses().getMainWorkflow().addTransitions(
                t1.getSource(), t1.getTarget());
        project.getMyProcesses().getMainWorkflow().addTransitions(
                t2.getSource(), t2.getTarget());

        assertNotNull(project.getMyProcesses().getMainWorkflow()
                .getTransitions());
        assertEquals(project.getMyProcesses().getMainWorkflow()
                .getTransitions().size(), 2);

    }

    public void testGenerationXPDL()
    {
    	String[] args = new String[0];
    	MainFrame owner = new MainFrame(true,true,args);
        Document document = new Document();
        Format format = Format.getPrettyFormat();
        format.setEncoding(XPDLProperty.XPDL_ENCODING);
        XMLOutputter sortie = new XMLOutputter(format);

        xpdlProject = new XPDLProEdProject(project,owner);
        xpdlProject.toXML(document);
        assertNotNull(xpdlProject);

        //outputString : return a string of all the document
        try
        {
            sortie.output(document, new FileOutputStream(fileName));

        }
        catch (IOException ioe)
        {
            System.err.println(ioe.getMessage());
        }

        document.removeContent();
    }

    public void testLoadXPDL()
    {
    	String[] args = new String[0];
    	MainFrame owner = new MainFrame(true,true,args);
        Format format = Format.getPrettyFormat();
        format.setEncoding(XPDLProperty.XPDL_ENCODING);
        XMLOutputter sortie = new XMLOutputter(format);

        Document doc = null;
        try
        {
            doc = Utils.loadXMLDocument(fileName);
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        if (doc != null)
        {
            try
            {
                sortie.output(doc, new FileOutputStream(fileName));
            }
            catch (IOException ioe)
            {
                System.err.println(ioe.getMessage());
            }

            // Load graph model
            ProEdProject newProject = XPDLProEdProject.toModel(doc,owner);
            assertNotNull(newProject);
            assertEquals(project, newProject);

        }

    }

    public static Test suite()
    {
        return new TestSuite(XPDLtest.class);
    }

}