/**
* LICENSE 
* Bonita
* Copyright (C) 2005 Bull S.A.
* Bull 68 route de versailles  78434 Louveciennes Cedex France
* Further information: bonita@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*/
package org.ow2.proed.chi.test.graphModel;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.ow2.proed.graphModel.AtomicActivity;
import org.ow2.proed.graphModel.Datafield;
import org.ow2.proed.graphModel.DatafieldType;
import org.ow2.proed.graphModel.PerformerAssignment;
import org.ow2.proed.graphModel.ProEdProject;
import org.ow2.proed.graphModel.ProcessActivity;
import org.ow2.proed.graphModel.Subflow;
import org.ow2.proed.graphModel.WorkflowProcess;
import org.ow2.proed.graphModel.WorkflowProcesses;


/**
 * AtomicActivityTest.java
 * Created on 11 juil. 2005
 * 
 * @author Ducreux Evelyne - Gonthier Manu�la
 * @version $Id: AtomicActivityTest.java 3943 2008-12-11 13:21:09Z vigneras $
 * <p>
 */
public class AtomicActivityTest extends TestCase
{
    private ProEdProject project;

    private AtomicActivity a1;

    private AtomicActivity a2;

    private AtomicActivity a3;

    private AtomicActivity a4;

    private AtomicActivity a5;

    private PerformerAssignment performerCallback;

    private PerformerAssignment performerProperty;

    private Datafield datafield;

    private ProcessActivity processActivity;

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception
    {
        project = new ProEdProject("Project1", false);
        WorkflowProcesses workflows = new WorkflowProcesses();
        WorkflowProcess mainWorkflow = new WorkflowProcess("project1");
        project.setMyProcesses(workflows);
        workflows.add(mainWorkflow);

        a1 = new AtomicActivity("Activit� 1",mainWorkflow);
        assertNotNull(a1);
        a2 = new AtomicActivity("Activit� 2", "2eme activit�",
                AtomicActivity.MANUAL_MODE,mainWorkflow);
        assertNotNull(a2);
        a3 = new AtomicActivity("Activit� 3", "super",mainWorkflow);
        assertNotNull(a3);
        a4 = new AtomicActivity("Activit� 4", "activit� 4",
                AtomicActivity.AUTO_MODE, AtomicActivity.TYPE_AND_JOIN,mainWorkflow);
        assertNotNull(a4);
        assertNull(a5);

        processActivity = new Subflow("subflow", "activity to copy",
                Subflow.TYPE_AND_JOIN,mainWorkflow);

        performerCallback = new PerformerAssignment(
                PerformerAssignment.CST_CUSTOM, "hero.hook.callback");
        performerProperty = new PerformerAssignment(
                PerformerAssignment.CST_VARIABLE, "datafield");

        datafield = new Datafield("datafield", "d1", DatafieldType.STRING, mainWorkflow);
    }

    public void testConstructAtomicActivity()
    {
        //test construction of atomic activity with processActivity
        a5 = AtomicActivity.constructAtomicActivity(processActivity);
        System.out.println("\nA5:" + a5.printtoString());
        System.out.println("\nProcessActivity:" + processActivity.printtoString());

    }

    public void testRemovePerformerAssignment()
    {
        a1.setPerformerAssignment(performerCallback);
        a1.removePerformerAssignment(datafield);
        assertNotNull(a1.getPerformerAssignment());
        a1.setPerformerAssignment(performerProperty);
        a1.removePerformerAssignment(datafield);
        assertNull(a1.getPerformerAssignment());

    }

    public static Test suite()
    {
        return new TestSuite(AtomicActivityTest.class);
    }
}