/**
 * LICENSE 
 * Bonita
 * Copyright (C) 2005 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.proed.chi.test.graphModel;

import java.util.Iterator;
import java.util.List;
import java.util.Set;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.ow2.proed.graphModel.AtomicActivity;
import org.ow2.proed.graphModel.BlockActivity;
import org.ow2.proed.graphModel.Datafield;
import org.ow2.proed.graphModel.DatafieldType;
import org.ow2.proed.graphModel.Participant;
import org.ow2.proed.graphModel.ProEdProject;
import org.ow2.proed.graphModel.ProcessActivity;
import org.ow2.proed.graphModel.Property;
import org.ow2.proed.graphModel.RouteActivity;
import org.ow2.proed.graphModel.Subflow;
import org.ow2.proed.graphModel.Transition;
import org.ow2.proed.graphModel.WorkflowProcess;
import org.ow2.proed.graphModel.WorkflowProcesses;
import org.ow2.proed.graphModel.exception.ParticipantTypeException;
import org.ow2.proed.graphModel.exception.TransitionException;
import org.ow2.proed.resources.PropertiesManager;

/**
 * SubflowTest.java Created on 11 juil. 2005
 * 
 * @author Ducreux Evelyne - Gonthier Manu�la
 * @version $Id: SubflowTest.java 3943 2008-12-11 13:21:09Z vigneras $
 *          <p>
 */
public class SubflowTest extends TestCase {

  private ProEdProject project;
  private Subflow subflow1;
  private Subflow subflow2;
  private Subflow subflow3;
  private AtomicActivity atomic1;
  private BlockActivity block2;
  private RouteActivity root3;
  private Subflow s4;
  // transitions to test
  private Transition t1;
  private Transition t2;
  private Transition t3;
  private Transition t4;
  private final PropertiesManager propManager = new PropertiesManager();
  private List listOfProperties;
  private Property prop1;
  private Property prop2;
  private Property propToPropagate;
  private Datafield datafield1;
  private Datafield datafield2;
  private Datafield datafield3;
  private Participant p1;
  private Participant p2;

  /*
   * @see TestCase#setUp()
   */
  @Override
  protected void setUp() throws Exception {
    this.project = new ProEdProject("Project1", false);
    final WorkflowProcesses workflows = new WorkflowProcesses();
    final WorkflowProcess mainWorkflow = new WorkflowProcess("project1");
    this.project.setMyProcesses(workflows);
    workflows.add(mainWorkflow);

    this.subflow1 = new Subflow("Subflow 1", mainWorkflow);
    Assert.assertNotNull(this.subflow1);
    System.out.println("A1:" + this.subflow1.printtoString());

    this.subflow2 = new Subflow("Subflow 2", "Description de subflow 2", mainWorkflow);
    Assert.assertNotNull(this.subflow2);
    System.out.println("A2:" + this.subflow2.printtoString());

    this.subflow3 = new Subflow("Subflow 3", "Description de subflow 3", ProcessActivity.TYPE_AND_JOIN, mainWorkflow);
    Assert.assertNotNull(this.subflow3);
    System.out.println("A3:" + this.subflow3.printtoString());

    this.subflow1.constructWorkflow();
    Assert.assertNotNull(this.subflow1.getWorkflow());

    // add activities and transition to this workflow
    // set up all process activities
    this.atomic1 = new AtomicActivity("Activit� 1", this.subflow1.getWorkflow());
    this.block2 = new BlockActivity("Activit� 2", this.subflow1.getWorkflow());
    this.root3 = new RouteActivity("Activit� 3", this.subflow1.getWorkflow());
    this.s4 = new Subflow("subflow 1", mainWorkflow);

    try {
      // set up good transitions
      this.t1 = new Transition(this.atomic1, this.block2, this.subflow1.getWorkflow());
      Assert.assertNotNull(this.t1);
      this.t2 = new Transition(this.block2, this.root3, this.subflow1.getWorkflow());
      Assert.assertNotNull(this.t2);
      this.t3 = new Transition(this.atomic1, this.s4, this.subflow1.getWorkflow());
      Assert.assertNotNull(this.t3);
    } catch (final TransitionException e2) {
      System.out.println(e2.getMessage());
    }

    // Add activities and transitions in subflow1
    this.subflow1.getWorkflow().addProcessActivity(this.atomic1);
    this.subflow1.getWorkflow().addProcessActivity(this.block2);
    this.subflow1.getWorkflow().addProcessActivity(this.root3);
    this.subflow1.getWorkflow().addProcessActivity(this.s4);

    this.subflow1.getWorkflow().addTransitions(this.t1.getSource(), this.t1.getTarget());
    this.subflow1.getWorkflow().addTransitions(this.t2.getSource(), this.t2.getTarget());
    this.subflow1.getWorkflow().addTransitions(this.t3.getSource(), this.t3.getTarget());

    this.subflow2.constructWorkflow();
    this.subflow2.getWorkflow().addProcessActivity(this.atomic1);
    this.subflow2.getWorkflow().addProcessActivity(this.block2);
    this.subflow2.getWorkflow().addProcessActivity(this.root3);
    this.subflow2.getWorkflow().addProcessActivity(this.s4);

    this.subflow2.getWorkflow().addTransitions(this.t1.getSource(), this.t1.getTarget());
    this.subflow2.getWorkflow().addTransitions(this.t2.getSource(), this.t2.getTarget());
    this.subflow2.getWorkflow().addTransitions(this.t3.getSource(), this.t3.getTarget());

    try {
      this.p1 = new Participant("Manue", Participant.CST_HUMAN, "Concepteur", this.subflow1.getWorkflow());
      this.p2 = new Participant("Jiap", Participant.CST_SYSTEM, "Concepteur", this.subflow1.getWorkflow());
    } catch (final ParticipantTypeException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

    this.subflow1.getWorkflow().getParticipants().add(this.p1);
    this.subflow1.getWorkflow().getParticipants().add(this.p2);

    this.datafield1 = new Datafield("data1", "d1", DatafieldType.STRING, this.subflow1.getWorkflow());
    this.datafield2 = new Datafield("data2", "d2", DatafieldType.STRING, this.subflow1.getWorkflow());
    this.datafield3 = new Datafield("data3", "d3", DatafieldType.STRING, this.subflow1.getWorkflow());
  }

  private void printTrajet(final WorkflowProcess workflow) {
    final Set allTransitions = workflow.getTransitions();
    for (final Iterator iter = allTransitions.iterator(); iter.hasNext();) {
      final Transition t = (Transition) iter.next();
      System.out.println(t.getSource() + "->" + t.getTarget());
    }
  }

  /*
   * Class under test for Datafield GetDatafield(String name)
   */
  public void testGetDatafield() {
    System.out.println("\n***********testGetDatafield\n");
    final Datafield data = this.subflow1.getWorkflow().getDatafieldByName("NotExist");
    Assert.assertNull(data);
  }

  /*
   * Class under test for Participant GetParticipant(String name)
   */
  public void testGetParticipant() {
    System.out.println("\n***********testGetParticipant\n");
    final Participant pPasTrouve = this.subflow1.getWorkflow().getParticipantByName("M");
    Assert.assertNull(pPasTrouve);
    final Participant pTrouve = this.subflow1.getWorkflow().getParticipantByName("Manue");
    Assert.assertNotNull(pTrouve);
  }

  public static Test suite() {
    return new TestSuite(SubflowTest.class);
  }

}